//  Copyright (C) 2002-2003 Intel Corporation, All Rights Reserved.
//  Permission is hereby granted to merge this program code with 
//  other program material to create a derivative work.  This 
//  derivative work may be distributed in compiled object form only.
//  Any other publication of this program, in any form, without the 
//  explicit permission of the copyright holder is prohibited.
//
//  Send questions and comments to erik.j.johnson@intel.com, 
//  aaron.kunze@intel.com

//-------------------------------------------------------------------
// hello_world.c - Chapter 4
// Reverses an array of 32-bit quantities in SRAM.
//

//-------------------------------------------------------------------
// Include files
#include "ixp.h"

//-------------------------------------------------------------------
// reverse_array
//
//    Description:
//       Reverses an array of 32-bit quantities in SRAM.
//
//    Parameters:
//      Outputs: n/a
//      In/Outs: n/a
//      Inputs: old - The original array
//				new - The array into which the reversed data should go.
//				size - The number of 32-bit quantities in old
//      Constants: n/a
//      Labels: n/a
//
//    Side effects: n/a
//
//    See also: n/a
//
//    Example Usage: reverse_array(old_array, new_array, 10)
//
void reverse_array(volatile int* old, 
				   volatile int* new, 
				   int size);

//-------------------------------------------------------------------
// main
//
//    Description:
//       Uses the reverse_array function to reverse an array of 32-bit 
//		 quantities in SRAM.
//
void main () {
	_declspec(shared sram) int old_array[] = 
	    { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
	_declspec(shared sram) int 
	    new_array[sizeof(old_array)/sizeof(int)];

	global_label("start_reverse");
	reverse_array(old_array, new_array, 
				  sizeof(old_array)/sizeof(int));
	global_label("end_reverse");
}

//-------------------------------------------------------------------
// exit
//
//    Description:
//       This is called when main() completes.
//
void exit (unsigned int arg) {
}

//-------------------------------------------------------------------
// reverse_array
//
//    Description:
//       Reverses an array of 32-bit quantities in SRAM.
//
//    Parameters:
//      Outputs: n/a
//      In/Outs: n/a
//      Inputs: old - The original array
//				new - The array into which the reversed data should go.
//				size - The number of 32-bit quantities in old
//      Constants: n/a
//      Labels: n/a
//
//    Side effects: n/a
//
//    See also: n/a
//
//    Example Usage: reverse_array(old_array, new_array, 10)
//
void reverse_array(volatile int* old, 
				   volatile int* new, 
				   int size) {
    int index = 0;

	for (index = 0; index < size; index++) {
		new[index] = old[size - index - 1];
	}
}
